/**
 ** rmp400CntrlTaskCodels.c
 **
 ** Codels used by the control task rmp400CntrlTask
 **
 ** Author: 
 ** Date: 
 **
 **/

#include <math.h>
#include <stdio.h>
#include <portLib.h>

#include "rmp400Const.h"
#include "server/rmp400Header.h"

/*------------------------------------------------------------------------
 * controlCmdParam  -  control codel of CONTROL request SetCmdParam
 *
 * Description:    
 * Report: OK
 *                 S_rmp400_BAD_CMD_PARAMETERS
 *
 * Returns:    OK or ERROR
 */

STATUS
controlCmdParam(GENPOS_CART_CMD_PARAM *cmd, int *report)
{
	/* ... add your code here ... */
	return OK;
}

/*------------------------------------------------------------------------
 * computeStopRef  -  control codel of CONTROL request Stop
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
computeStopRef(int *report)
{
	GENPOS_CART_STATE *robotId;           /* Config robot */
	GENPOS_CART_REF *refId;               /* Consigne */
	double dtArret, dtRetard;  /* Temps prise en compte et temps deceleration */
	double dSArret, dTArret;   /* Longueurs lin et ang de decel */
	double angleFinal;         /* Angle a l'arrivee */
	double period;
	
	robotId = &rmp400DataStrId->robot;
	refId = &rmp400DataStrId->ref;
	
	/* Longueur d'arret */
	period = EXEC_TASK_PERIOD(RMP400_MOTIONTASK_NUM);
	dtArret = MAX (fabs(robotId->v) / RMP400_ACCEL_LIN_MAX,
		       fabs(robotId->w) / RMP400_ACCEL_ANG_MAX);
	dtRetard =  1 * period;
	dSArret = robotId->v * (dtArret / 2 + dtRetard);
	dTArret = robotId->w * (dtArret / 2 + dtRetard);
	angleFinal = robotId->theta + dTArret;
	
	/* Consigne */
	refId->x = robotId->xRob + dSArret * cos (angleFinal);
	refId->y = robotId->yRob + dSArret * sin (angleFinal);
	refId->v = 0. ;
	refId->w = 0. ;
	return(OK);
}

/*------------------------------------------------------------------------
 * trackStart  -  control codel of EXEC request Track
 *
 * Description:    Check tracking mode compatibility and start tracking 
 *                 if possible
 * Report: OK
 *                 S_rmp400_POSTER_NOT_FOUND
 *                 S_rmp400_BAD_REF
 *                 S_rmp400_CMD_STOP_TRACK
 *                 S_rmp400_MOTORS_OFF
 *
 * Returns:    OK or ERROR
 */

STATUS
trackStart(RMP400_TRACK_PARAMS *params, int *report)
{
	RMP400_STATUS_STR *status = &SDI_F->status;
	int retval = OK;

	/* Check if state permits motion */
	if (status->rs_mode == RMP400_MOTORS_OFF || 
	    status->rs_mode == RMP400_EMERGENCY) {
		*report = S_rmp400_MOTORS_OFF;
		return ERROR;
	}
	/* Start correct tracking mode */
	*report = 0;
	switch (params->mode) {
	case GENPOS_NO_TRACKING:
		status->rs_mode = RMP400_IDLE;
		rmp400DataStrId->trackStr.trackMode = params->mode;
		*report = S_rmp400_CMD_STOP_TRACK;
		retval = ERROR;
		break;
	case GENPOS_TRACK_POS:
		status->rs_mode = RMP400_TRACK_POSITION;
		rmp400DataStrId->trackStr.trackMode = params->mode;
		break;
	case GENPOS_TRACK_CONFIG:
		status->rs_mode = RMP400_TRACK_CONFIG;
		rmp400DataStrId->trackStr.trackMode = params->mode;
		break;
	case GENPOS_TRACK_SPEED:
		status->rs_mode= RMP400_TRACK_SPEED;
		rmp400DataStrId->trackStr.trackMode = params->mode;
		break;
	default:
		*report = S_rmp400_BAD_REF;
		retval = ERROR;
	}
	return retval;
}


