/**
 ** rmp400TrackTaskCodels.c
 **
 ** Codels called by execution task rmp400TrackTask
 **
 ** Author: 
 ** Date: 
 **
 **/
#include <math.h>
#include <portLib.h>
#include <h2mathLib.h>

#include "server/rmp400Header.h"
#include "rmp400Const.h"

/*------------------------------------------------------------------------
 * TrackSpeed
 *
 * Description: 
 *
 * Reports:      OK
 *              S_rmp400_POSTER_NOT_FOUND
 *              S_rmp400_BAD_REF
 *              S_rmp400_CMD_STOP_TRACK
 */

/* findTrackedPoster  -  codel START of TrackSpeed
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
findTrackedPoster(RMP400_TRACK_PARAMS *params, int *report)
{
	char *trackedPoster = (char *)params->posterName.name;
	POSTER_ID posterTrackedId;
	
	/* Poster Identifier */
	if (posterFind((char *) trackedPoster, &posterTrackedId) != OK)  {
		*report = S_rmp400_POSTER_NOT_FOUND;
		return(END); 
	}
	rmp400DataStrId->trackStr.posterId = (int) posterTrackedId;
	printf("rmp400 tracking on %s\n", (char *)trackedPoster);
	return(EXEC);
}

static ACTIVITY_EVENT
pumpConfigReference(int *report)
{
	GENPOS_CART_REF rmp400Orders;
	GENPOS_CART_REF *refId = &rmp400DataStrId->ref;
	GENPOS_CART_STATE *configId = &rmp400DataStrId->robot;

	/* Lire les consignes sur le poster externe */
	if (posterRead ((POSTER_ID)rmp400DataStrId->trackStr.posterId, 0, 
		(void *) &rmp400Orders,
		sizeof (GENPOS_CART_REF)) == sizeof (GENPOS_CART_REF)) {
		
		/* Verifier la validite des arguments */
		if (rmp400Orders.v > RMP400_VMAX ||
		    rmp400Orders.w > RMP400_WMAX) {
			
			/* On met une consigne de vitesse nulle sur le point
			 courant. XXXX ouch. */
			refId->v = 0.;
			refId->w = 0.;
			refId->x = configId->xRef;
			refId->y = configId->yRef;
			refId->theta = configId->theta;
			refId->dataType = GENPOS_POS_AND_SPEED_DATA;
			*report = S_rmp400_BAD_REF;
			return (END);
		}
		
		/* Transmettre la consigne */
		else {
			refId->backFlag = (rmp400Orders.v > 0 ?
			    GENPOS_FORWARD_MOTION : GENPOS_BACKWARD_MOTION);
			refId->v = fabs(rmp400Orders.v);
			refId->vt = 0;
			refId->w = rmp400Orders.w;
			refId->vmax = rmp400Orders.vmax;
			refId->wmax = rmp400Orders.wmax;
			refId->linAccelMax = rmp400Orders.linAccelMax;
			refId->angAccelMax = rmp400Orders.angAccelMax;
			refId->x = rmp400Orders.x;
			refId->y = rmp400Orders.y;
			refId->theta =  angleLimit(rmp400Orders.theta);
			refId->dataType = GENPOS_POS_AND_SPEED_DATA;
			return(EXEC);
		} 
	}
	*report = errnoGet();
	return(END);
}

static ACTIVITY_EVENT
pumpSpeedReference(int *report)
{
	GENPOS_CART_SPEED rmp400Orders;
	GENPOS_CART_REF *refId = &rmp400DataStrId->ref;
	GENPOS_CART_STATE *configId = &rmp400DataStrId->robot;

	/* Lire les consignes sur le poster externe */
	if (posterRead ((POSTER_ID)rmp400DataStrId->trackStr.posterId, 0, 
		(void *) &rmp400Orders,
		sizeof (GENPOS_CART_SPEED)) == sizeof (GENPOS_CART_SPEED)) {
		
		/* Verifier la validite des arguments */
		if (rmp400Orders.v > RMP400_VMAX ||
		    rmp400Orders.w > RMP400_WMAX) {
			
			/* On met une consigne de vitesse nulle sur le point
			 courant. XXXX ouch. */
			refId->v = 0.;
			refId->w = 0.;
			refId->dataType = GENPOS_SPEED_DATA;
			*report = S_rmp400_BAD_REF;
			return (END);
		}
		
		/* Transmettre la consigne */
		else {
			refId->backFlag = (rmp400Orders.v > 0 ?
			    GENPOS_FORWARD_MOTION : GENPOS_BACKWARD_MOTION);
			refId->v = fabs(rmp400Orders.v);
			refId->vt = 0;
			refId->w = rmp400Orders.w;
			refId->vmax = rmp400Orders.vmax;
			refId->wmax = rmp400Orders.wmax;
			refId->linAccelMax = rmp400Orders.linAccelMax;
			refId->angAccelMax = rmp400Orders.angAccelMax;
			refId->dataType = GENPOS_SPEED_DATA;
			return(EXEC);
		} 
	}
	*report = errnoGet();
	return(END);
}

/* pumpReference  -  codel EXEC of TrackSpeed
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
pumpReference(RMP400_TRACK_PARAMS *params, int *report)
{
	GENPOS_TRACK *track = &SDI_F->trackStr;

	switch (track->trackMode) {
	case GENPOS_TRACK_POS:
		return pumpConfigReference(report);
	case GENPOS_TRACK_CONFIG:
		return pumpConfigReference(report);
	case GENPOS_TRACK_SPEED:
		return pumpSpeedReference(report);
	default:
		*report = EXEC_TASK_BILAN(RMP400_MOTIONTASK_NUM);
		return(END);

	}
}

/* smoothStopTrack  -  codel END of TrackSpeed
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
smoothStopTrack(RMP400_TRACK_PARAMS *params, int *report)
{
	GENPOS_CART_REF *refId;	       /* Id structure de consignes */
	GENPOS_CART_STATE *robotId;	/* Id config du robot */
	double dtArret;                /* Temps d'arret */
	double dtRetard;               /* Temps de prise en compte de la consigne */
	double dsArret;                /* Longueur d'arret */
	double dtLoco;                 /* Periode d'asserv de la loco */
	double angle;                  /* Moyenne entre angles d'init et d'arret */
	double theta;

	printf("rmp400 smoothStopTrack\n");
	/* Obtenir les pointeurs vers les structures de donnees de la locomotion */
	robotId = &(rmp400DataStrId->robot);
	refId = &rmp400DataStrId->ref;
	
	/* Determination de la periode d'asserv de la loco */
	dtLoco = EXEC_TASK_PERIOD(RMP400_TRACKTASK_NUM);
	
	/* Temps d'arret minimal (tps de decel) */
	dtArret = MAX (fabs(robotId->v) / RMP400_ACCEL_LIN_MAX,
	    fabs(robotId->w) / RMP400_ACCEL_ANG_MAX);
	
	/* Temps de prise en compte (v = cst) */
	dtRetard =  1 * dtLoco;
	
	/* Angle final (avec periode de prise en compte) */
	theta = robotId->theta + robotId->w * (dtArret / 2 + dtRetard);
	
	/* Angle moyen */
	angle = (robotId->theta + theta) / 2;
	
	/* Longueur d'arret */
	dsArret = robotId->v * (dtArret / 2 + dtRetard);
	
	/* Prevision position du centre */
	refId->x = robotId->xRob + dsArret * cos (angle);
	refId->y = robotId->yRob + dsArret * sin (angle);
	
	/* Vitesses de consigne nulles */
	refId->v = 0. ;
	refId->w = 0. ;
	
	/* Indiquer l'arret du tracking */
	refId->dataType = GENPOS_POS_AND_SPEED_DATA;
	rmp400DataStrId->trackStr.trackMode = GENPOS_NO_TRACKING;
	return (OK);
	
}

/* smoothStopTrack  -  codel INTER of TrackSpeed
   see above (request TrackSpeed) */

