/*
 * Copyright (c) 2009 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef _RMP400STRUCT_H
#define _RMP400STRUCT_H

typedef enum RMP400_MODE {
	RMP400_IDLE = 0,
	RMP400_MOTORS_OFF,
	RMP400_MANUAL,
	RMP400_TRACK_POSITION,
	RMP400_TRACK_CONFIG,
	RMP400_TRACK_SPEED,
	RMP400_EMERGENCY
} RMP400_MODE;

typedef struct RMP400_DATA_STR {
	double pitch_angle;
	double pitch_rate;
	double roll_angle;
	double roll_rate;
	double lw_velocity;
	double rw_velocity;
	double yaw_rate;
	short servo_frames;
	double integrated_left_wheel;
	double integrated_right_wheel;
	double integrated_fore_aft;
	double integrated_yaw;
	double left_torque;
	double right_torque;
	short operational_mode;
	short controller_gain_schedule;
	double ui_voltage;
	double powerbase_voltage;
	double velocity_command;
	double turn_command;
} RMP400_DATA_STR;
       
typedef struct RMP400_STATUS_STR {
	RMP400_MODE rs_mode;
	RMP400_DATA_STR rs_data[2];
} RMP400_STATUS_STR;

/* Kinematics of the robot */
typedef struct RMP400_KINEMATICS_STR {
	double leftWheelRadius;	
	double rightWheelRadius;
	double axisWidth;
} RMP400_KINEMATICS_STR;

/* Coefficient proportionnel a la vitesse pour calculer l'erreur sur 
   le deplacement des roues */
typedef struct RMP400_VAR_PARAMS {
	double coeffLin;
	double coeffAng;
	double coeffLinAng;
} RMP400_VAR_PARAMS;

/* Track parameters */
typedef struct RMP400_TRACK_PARAMS {
	GENPOS_POSTER_NAME posterName;
	GENPOS_TRACK_MODE mode;
} RMP400_TRACK_PARAMS;

#endif
